@@ -1,4 +1,15 @@
 (function () {
+    const THEME_OPTIONS = [
+        { id: 'dark-tritanopia', label: 'Dark Tritanopia' },
+        { id: 'soft-dark', label: 'Soft Dark' },
+        { id: 'dark-purple', label: 'Dark Purple' },
+        { id: 'dark', label: 'Dark' },
+        { id: 'light', label: 'Light' },
+        { id: 'white', label: 'White' },
+        { id: 'solarized', label: 'Solarized' },
+        { id: 'mono-blue', label: 'Monochrome Blue' },
+    ];
+
     let activeGroupId = null;
     const sidebarLinkLookup = new Map();
     const activeSidebarLinks = new Map();
@@ -6,7 +17,7 @@
     let scrollRaf = null;
 
     document.addEventListener('DOMContentLoaded', () => {
-        initThemeToggle();
+        initThemePicker();
         initSidebarNav();
         initTabs();
         initScrollHighlighting();
@@ -16,28 +27,126 @@
         hydrateSwitches();
     });
 
-    function initThemeToggle() {
+    function initThemePicker() {
         const root = document.documentElement;
-        const toggle = document.querySelector('[data-theme-toggle]');
+        const panel = document.querySelector('[data-theme-panel]');
         const storedTheme = window.localStorage.getItem('uime-theme');
-        const initialTheme = storedTheme || root.dataset.theme || 'dark';
-        root.dataset.theme = initialTheme;
+        const fallbackTheme = root.dataset.theme || THEME_OPTIONS[0].id;
+        const initialTheme = THEME_OPTIONS.some(({ id }) => id === storedTheme) ? storedTheme : fallbackTheme;
+        const swatches = document.querySelectorAll('[data-theme-option]');
+        let isPanelManuallyExpanded = false;
+        const preview = panel?.querySelector('[data-theme-display]');
 
-        const updateLabel = () => {
-            if (!toggle) {
+        if (panel && !panel.getAttribute('data-panel-state')) {
+            panel.setAttribute('data-panel-state', 'collapsed');
+        }
+
+        function handleOutsideClick(event) {
+            if (!panel || panel.contains(event.target)) {
                 return;
             }
-            const next = root.dataset.theme === 'dark' ? 'Switch to light mode' : 'Switch to dark mode';
-            toggle.setAttribute('aria-label', next);
+            collapsePanel();
+        }
+
+        function collapsePanel() {
+            if (!panel) {
+                return;
+            }
+            panel.setAttribute('data-panel-state', 'collapsed');
+            if (isPanelManuallyExpanded) {
+                document.removeEventListener('pointerdown', handleOutsideClick);
+                isPanelManuallyExpanded = false;
+            }
+        }
+
+        function expandPanel(manual = false) {
+            if (!panel) {
+                return;
+            }
+            panel.setAttribute('data-panel-state', 'expanded');
+            if (manual && !isPanelManuallyExpanded) {
+                isPanelManuallyExpanded = true;
+                document.addEventListener('pointerdown', handleOutsideClick);
+            }
+        }
+
+        const applyTheme = (theme) => {
+            const option = THEME_OPTIONS.find(({ id }) => id === theme);
+            if (!option) {
+                return;
+            }
+            root.dataset.theme = option.id;
+            window.localStorage.setItem('uime-theme', option.id);
+            panel?.setAttribute('data-current-theme', option.id);
+            preview?.setAttribute('data-theme-option', option.id);
+            swatches.forEach((swatch) => {
+                const isActive = swatch.dataset.themeOption === option.id;
+                swatch.classList.toggle('is-selected', isActive);
+                swatch.setAttribute('aria-checked', String(isActive));
+                swatch.tabIndex = isActive ? 0 : -1;
+            });
+            if (panel && isPanelManuallyExpanded && !panel.matches(':hover')) {
+                collapsePanel();
+            }
         };
 
-        updateLabel();
+        applyTheme(initialTheme);
+        panel?.setAttribute('data-current-theme', initialTheme);
 
-        toggle?.addEventListener('click', () => {
-            const nextTheme = root.dataset.theme === 'dark' ? 'light' : 'dark';
-            root.dataset.theme = nextTheme;
-            window.localStorage.setItem('uime-theme', nextTheme);
-            updateLabel();
+        if (panel) {
+            panel.addEventListener('mouseenter', () => {
+                if (!isPanelManuallyExpanded) {
+                    expandPanel();
+                }
+            });
+
+            panel.addEventListener('mouseleave', () => {
+                if (isPanelManuallyExpanded || panel.matches(':focus-within')) {
+                    return;
+                }
+                collapsePanel();
+            });
+
+            panel.addEventListener('focusin', () => {
+                expandPanel();
+            });
+
+            panel.addEventListener('focusout', (event) => {
+                const nextTarget = event.relatedTarget;
+                if (nextTarget && panel.contains(nextTarget)) {
+                    return;
+                }
+                if (isPanelManuallyExpanded || panel.matches(':hover')) {
+                    return;
+                }
+                collapsePanel();
+            });
+
+            panel.addEventListener('click', (event) => {
+                if (panel.getAttribute('data-panel-state') !== 'collapsed') {
+                    return;
+                }
+                expandPanel(true);
+                const swatch = event.target.closest('[data-theme-option]');
+                if (swatch) {
+                    event.preventDefault();
+                    event.stopPropagation();
+                    swatch.focus();
+                }
+            }, true);
+        }
+
+        swatches.forEach((swatch) => {
+            const theme = swatch.dataset.themeOption;
+            swatch.setAttribute('role', 'radio');
+            swatch.setAttribute('aria-label', THEME_OPTIONS.find(({ id }) => id === theme)?.label || theme);
+            swatch.addEventListener('click', () => applyTheme(theme));
+            swatch.addEventListener('keydown', (event) => {
+                if (event.key === 'Enter' || event.key === ' ') {
+                    event.preventDefault();
+                    applyTheme(theme);
+                }
+            });
         });
     }
 
